@@ -513,7 +513,12 @@ func Keybags(c *Config) (fname string, err error) {
 			return "", fmt.Errorf("failed to open IPSW: %v", err)
 		}
 		defer zr.Close()
-		kbags, err = img4.ParseZipKeyBags(zr.File, i, c.Pattern)
+		kbags, err = img4.ParseZipKeyBags(zr.File, &img4.MetaData{
+			Type:                  i.Plists.Type,
+			ProductVersion:        i.Plists.BuildManifest.ProductVersion,
+			ProductBuildVersion:   i.Plists.BuildManifest.ProductBuildVersion,
+			SupportedProductTypes: i.Plists.Restore.SupportedProductTypes,
+		}, c.Pattern)
 		if err != nil {
 			return "", fmt.Errorf("failed to parse im4p kbags: %v", err)
 		}
@@ -526,7 +531,12 @@ func Keybags(c *Config) (fname string, err error) {
 		if err != nil {
 			return "", err
 		}
-		kbags, err = img4.ParseZipKeyBags(zr.File, i, c.Pattern)
+		kbags, err = img4.ParseZipKeyBags(zr.File, &img4.MetaData{
+			Type:                  i.Plists.Type,
+			ProductVersion:        i.Plists.BuildManifest.ProductVersion,
+			ProductBuildVersion:   i.Plists.BuildManifest.ProductBuildVersion,
+			SupportedProductTypes: i.Plists.Restore.SupportedProductTypes,
+		}, c.Pattern)
 		if err != nil {
 			return "", fmt.Errorf("failed to parse im4p kbags: %v", err)
 		}
