@@ -719,7 +719,7 @@ func getAbbreviatedDevListFolder(devices []string) string {
 }
 
 // Parse parses plist files in a local ipsw file
-func Parse(ipswPath string) (*Info, error) {
+func Parse(ipswPath string, keys ...string) (*Info, error) {
 	var err error
 
 	i := &Info{}
@@ -728,7 +728,7 @@ func Parse(ipswPath string) (*Info, error) {
 	if err != nil {
 		return nil, fmt.Errorf("failed to parse plists: %v", err)
 	}
-	i.DeviceTrees, err = devicetree.Parse(ipswPath)
+	i.DeviceTrees, err = devicetree.Parse(ipswPath, keys...)
 	if err != nil {
 		if errors.Is(err, devicetree.ErrEncryptedDeviceTree) { // FIXME: this is a hack to avoid stopping the parsing of the metadata info
 			log.Error(err.Error())
